@@ -31,16 +31,16 @@ class AlphaShape(object):
 
         raise ValueError("dimension of a point cloud must be 2 or 3")
 
-    def create_filtration(self, no_square=False, symbols=None, save_boundary_map=False,
+    def create_filtration(self, squared=True, symbols=None, save_boundary_map=False,
                           save_phtrees=False):
         simplices = [
-            Simplex(alpha_simplex, i, no_square)
+            Simplex(alpha_simplex, i, squared)
             for (i, alpha_simplex)
             in enumerate(sorted(self.simplices, key=attrgetter("birth_radius")))
         ]
 
         dict_simplices = {simplex.key: simplex for simplex in simplices}
-        return AlphaFiltration(self.coordinates, self.periodicity, simplices, dict_simplices,
+        return AlphaFiltration(self.coordinates, self.weights, squared, self.periodicity, simplices, dict_simplices,
                                self.dim, symbols, save_boundary_map, save_phtrees)
 
     def subsets(self):
@@ -67,7 +67,7 @@ class AlphaShape(object):
             return groups
 
         return {
-            group_name: AlphaSubset(group_name, self.coordinates, self.periodicity, simplices, self.dim)
+            group_name: AlphaSubset(group_name, self.coordinates, self.weights, self.periodicity, simplices, self.dim)
             for (group_name, simplices)
             in group_by(self.simplices, group_of_simplex).items()
         }
@@ -94,6 +94,7 @@ class AlphaShape(object):
 class AlphaShape3(AlphaShape3Base, AlphaShape):
     def __init__(self, points, weighted, rel_homology):
         super().__init__(points, weighted, rel_homology)
+        self.weights = points[:, 3] if weighted else None
         self.coordinates = points[:, 0:3]
         self.simplices = self.vertices() + self.edges() + self.facets() + self.cells()
 
@@ -109,6 +110,7 @@ class AlphaShape3(AlphaShape3Base, AlphaShape):
 class AlphaShape2(AlphaShape2Base, AlphaShape):
     def __init__(self, points, weighted, rel_homology):
         super().__init__(points, weighted, rel_homology)
+        self.weights = points[:, 3] if weighted else None
         self.coordinates = points[:, 0:2]
         self.simplices = self.vertices() + self.edges() + self.faces()
 
@@ -127,6 +129,7 @@ class PeriodicAlphaShape3(PeriodicAlphaShape3Base, AlphaShape):
         super().__init__(points, weighted, rel_homology,
                          xmin, xmax, ymin, ymax, zmin, zmax)
         self.coordinates = points[:, 0:3]
+        self.weights = points[:, 3] if weighted else None
         self.simplices = self.vertices() + self.edges() + self.facets() + self.cells()
         self.periodicity = periodicity
 
@@ -136,11 +139,12 @@ class PeriodicAlphaShape3(PeriodicAlphaShape3Base, AlphaShape):
 
 
 class AlphaSubset(AlphaShape):
-    def __init__(self, group_name, points, periodicity, simplices, dim):
+    def __init__(self, group_name, points, weights, periodicity, simplices, dim):
         self.group_name = group_name
         self.periodicity = periodicity
         self.simplices = simplices
         self.coordinates = points
+        self.weights = weights
         self.dim = dim
 
     def isacyclic(self):
@@ -150,10 +154,10 @@ class AlphaSubset(AlphaShape):
 class Simplex(object):
     """A class representing simplex"""
 
-    def __init__(self, alpha_simplex, index, no_square=False):
+    def __init__(self, alpha_simplex, index, squared=True):
         self.index = index  # NOTE: This index is different from vertex's index
         self.key = AlphaShape.simplex_key(alpha_simplex)
-        self.birth_radius = self.normalize_radius(alpha_simplex.birth_radius, no_square)
+        self.birth_radius = self.normalize_radius(alpha_simplex.birth_radius, squared)
         self.isvertex = alpha_simplex.isvertex()
 
     def __repr__(self):
@@ -175,11 +179,8 @@ class Simplex(object):
         return list(zip(plusminus_alternative(len(unsigned)), unsigned))
 
     @staticmethod
-    def normalize_radius(r, no_square):
-        if no_square:
-            return math.copysign(math.sqrt(abs(r)), r)
-        else:
-            return r
+    def normalize_radius(r, squared):
+        return r if squared else math.copysign(math.sqrt(abs(r)), r)
 
     @property
     def dim(self):
@@ -188,11 +189,13 @@ class Simplex(object):
 
 
 class AlphaFiltration(FiltrationWithIndex):
-    def __init__(self, points, periodicity, simplices, dict_simplices, dim, symbols,
+    def __init__(self, points, point_weights, squared, periodicity, simplices, dict_simplices, dim, symbols,
                  save_boundary_map, save_phtrees):
         """
         Args:
         points -- list of N-d point coordinates
+        point_weights -- weights of points, None if weight is not given
+        squared -- whether all radii are squared
         periodicity -- None or list of (min, max)
         simplices -- list of simplices, must be sorted by their birth_radius
         dict_simplices -- dictiorary: simplex.key -> simplex
@@ -202,6 +205,8 @@ class AlphaFiltration(FiltrationWithIndex):
         save_phTrees -- bool
         """
         self.points = points
+        self.point_weights = point_weights
+        self.squared = squared
         self.periodicity = periodicity
         self.simplices = simplices
         self.dict_simplices = dict_simplices
@@ -210,12 +215,16 @@ class AlphaFiltration(FiltrationWithIndex):
         self.save_boundary_map = save_boundary_map
         self.save_phtrees = save_phtrees
 
+    def alpha_information_chunk(self):
+        return AlphaInformationChunk(self.points.shape[0], self.periodicity,
+                                     self.points_weighted, self.squared)
+
     def compute_pdgm(self, f, algorithm=None, output_suppl_info=True):
         writer = PDGMWriter(f, "alpha", self.dim)
         pairs, boundary_map_byte_sequence = self.compute_pairs(algorithm)
         writer.save_pairs(pairs, self.index_to_level, output_suppl_info)
-        writer.append_chunk(AlphaInformationChunk(self.points.shape[0],
-                                                  self.periodicity))
+        writer.append_chunk(self.alpha_information_chunk())
+
         if output_suppl_info:
             writer.append_simple_chunk("index_to_level", self.index_to_level)
             writer.append_simple_chunk("vertex_symbols", self.symbols)
@@ -241,6 +250,10 @@ class AlphaFiltration(FiltrationWithIndex):
     def index_to_simplex(self):
         return [list(simplex.key) for simplex in self.simplices]
 
+    @property
+    def points_weighted(self):
+        return self.point_weights is not None
+
     @property
     def index_map(self):
         return IndexMapForAlphaFiltration(
