@@ -1,5 +1,3 @@
-
-import sys
 import struct
 import io
 import os
@@ -7,15 +5,20 @@ import itertools
 from operator import attrgetter
 import warnings
 import enum
+import math
 
 import numpy as np
-from scipy.ndimage.filters import gaussian_filter
 import msgpack
 
 from homcloud.index_map import IndexMap, MapType
+from homcloud.pdgm_format import (
+    PDGMReader,
+    MAGIC_HEADER as PDGM_MAGIC_HEADER,
+    MAGIC_HEADER_LENGTH as PDGM_MAGIC_HEADER_LENGTH
+)
 
 
-MINUS_INF = -sys.float_info.max
+MINUS_INF = -math.inf
 
 
 class PD(object):
@@ -28,6 +31,7 @@ class PD(object):
     Instance variables:
     births -- birth times, 1D ndarray of floats
     deaths -- death times, 1D ndarray of floats
+    essential_births -- essential birth times, 1D ndarray of floats (deaths are infinity)
     birth_indices -- index birth times, 1D ndarray of int
     death_indices -- index death times, 1D ndarray of int
     birth_positions -- list of birth pixels or birth simplices
@@ -36,12 +40,14 @@ class PD(object):
     index_map -- index map defined in index_map module
     sign_flipped -- True if superlevel filtration, otherwise False
     """
-    def __init__(self, births, deaths, degree=None, essential_births=np.empty((0,)),
-                 boundary_map_dict=None, sign_flipped=False):
+
+    def __init__(self, births, deaths, degree=None,
+                 essential_births=np.empty((0,)), boundary_map_dict=None,
+                 sign_flipped=False):
         self.births = births
         self.deaths = deaths
         self.essential_births = essential_births
-        self.birth_indices = self.death_indices = None
+        self.birth_indices = self.death_indices = self.essential_birth_indices = None
         self.birth_positions = None
         self.death_positions = None
         self.filtration_type = None
@@ -134,15 +140,8 @@ class PD(object):
             return PD.empty_pd()
         births = np.concatenate([pd.births for pd in pds])
         deaths = np.concatenate([pd.deaths for pd in pds])
-        new_pd = PD(births, deaths)
-        new_pd.filtration_type = pds[0].filtration_type
-        if new_pd.filtration_type:
-            new_pd.birth_positions = list(itertools.chain.from_iterable([
-                pd.birth_positions for pd in pds
-            ]))
-            new_pd.death_positions = list(itertools.chain.from_iterable([
-                pd.death_positions for pd in pds
-            ]))
+        ess_births = np.concatenate([pd.essential_births for pd in pds])
+        new_pd = PD(births, deaths, None, ess_births)
 
         return new_pd
 
@@ -266,8 +265,39 @@ class PD(object):
         return pd
 
     @staticmethod
-    def load(typestr, path, degree, negate=False):
-        filetype = PD.estimate_filetype(typestr, path)
+    def load_from_pdgm(f, degree):
+        reader = PDGMReader(f)
+        births, deaths, ess_births = reader.load_pd_chunk("pd", degree)
+        pd = PD(np.array(births, dtype=float), np.array(deaths, dtype=float),
+                degree, np.array(ess_births, dtype=float))
+        birth_indices, death_indices, ess_birth_indices = reader.load_pd_chunk(
+            "indexed_pd", degree
+        )
+        if birth_indices is not None:
+            pd.birth_indices = np.array(birth_indices, dtype=int)
+            pd.death_indices = np.array(death_indices, dtype=int)
+            pd.essential_birth_indices = np.array(ess_birth_indices, dtype=int)
+
+        if reader.filtration_type == "abstract":
+            pd.filtration_type = MapType.abstract
+            symbols = reader.load_simple_chunk("index_to_symbol")
+            pd.birth_positions = [symbols[i] for i in pd.birth_indices]
+            pd.death_positions = [symbols[i] for i in pd.death_indices]
+            pd.essential_birth_positions = [symbols[i] for i in pd.essential_birth_indices]
+        if reader.filtration_type == "rips":
+            pd.filtration_type = MapType.rips
+
+        pd.sign_flipped = reader.metadata["sign_flipped"]
+        return pd
+
+    @staticmethod
+    def load_from_pdgm_file(path, degree):
+        with open(path, "rb") as f:
+            return PD.load_from_pdgm(f, degree).set_path(path)
+
+    @staticmethod
+    def load(filetype, path, degree, negate=False):
+        filetype = PD.estimate_filetype(filetype, path)
         if filetype == PD.FileType.dipha_diagram:
             pd = PD.load_from_diphafile(path, degree)
         elif filetype == PD.FileType.idipha_diagram:
@@ -278,6 +308,8 @@ class PD(object):
             pd = PD.load_from_pmtfile(path, degree)
         elif filetype == PD.FileType.p2mt:
             pd = PD.load_from_p2mtfile(path, degree)
+        elif filetype == PD.FileType.pdgm:
+            pd = PD.load_from_pdgm_file(path, degree)
         else:
             raise ValueError("Unkown file format: {}".format(path))
         if negate:
@@ -324,6 +356,7 @@ class PD(object):
             "text": PD.FileType.text,
             "pmt": PD.FileType.persistence_merge_tree,
             "p2mt": PD.FileType.p2mt,
+            "pdgm": PD.FileType.pdgm,
         }
         TABLE_EXTENSION = {
             ".idiagram": PD.FileType.idipha_diagram,
@@ -331,10 +364,19 @@ class PD(object):
             ".pmt": PD.FileType.persistence_merge_tree,
             ".p2mt": PD.FileType.p2mt,
             ".txt": PD.FileType.text,
+            ".pdgm": PD.FileType.pdgm,
         }
 
+        def check_file_header(path):
+            with open(path, "rb") as f:
+                if f.read(PDGM_MAGIC_HEADER_LENGTH) == PDGM_MAGIC_HEADER:
+                    return PD.FileType.pdgm
+                else:
+                    return None
+
         _, ext = os.path.splitext(path)
         return(TABLE_TYPENAME.get(typestr) or
+               check_file_header(path) or
                TABLE_EXTENSION.get(ext) or
                PD.FileType.unknown)
 
@@ -433,6 +475,7 @@ class PD(object):
         idipha_diagram = 2
         persistence_merge_tree = 3
         p2mt = 4
+        pdgm = 5
         unknown = -1
 
 
@@ -456,231 +499,3 @@ class Pair(object):
 
     def __repr__(self):
         return "Pair({})".format(self.display_str())
-
-
-class Histogram(object):
-    """A histogram, normally used for representing persistence diagrams.
-    The PDHistogram subclass is used for this purpose.
-
-    This class is also used for representing vectorized histogram.
-    """
-    def __init__(self, values, xedges, yedges, sign_flipped=False):
-        self.values = values
-        self.xedges = xedges
-        self.yedges = yedges
-        self.values_as_float = values.astype(float)
-        self.sign_flipped = sign_flipped
-
-    def xy_extent(self):
-        """Return [xmin, xmax, ymin, ymax].
-        """
-        return [self.xedges[0], self.xedges[-1], self.yedges[0], self.yedges[-1]]
-
-    def x_range(self):
-        """Return the range of the histogram along with x-axis as a pair of (min, max).
-        """
-        return (self.xedges[0], self.xedges[-1])
-
-    def y_range(self):
-        """Return the range of the histogram along with y-axis as a pair of (min, max).
-        """
-        return (self.yedges[0], self.yedges[-1])
-
-    def x_bins(self):
-        """Return the number of bins of x-axis.
-        """
-        return self.xedges.size - 1
-
-    def y_bins(self):
-        """Return the number of bins of y-axis.
-        """
-        return self.yedges.size - 1
-
-    def vectorize(self):
-        """Return vectorized histogram as a 1D ndarray of floats.
-        """
-        return self.values[self.vectorize_mask()]
-
-    def vectorize_mask(self):
-        """Internally used by vectorize().
-        """
-        if self.sign_flipped:
-            return self.xedges_of_bins() > self.yedges_of_bins()
-        else:
-            return self.xedges_of_bins() < self.yedges_of_bins()
-
-    def xedges_of_bins(self):
-        """Internally used by vectorize().
-        """
-        xedges = self.xedges[1:] if self.sign_flipped else self.xedges[:-1]
-        return np.repeat(xedges.reshape((1, self.x_bins())), self.y_bins(), axis=0)
-
-    def yedges_of_bins(self):
-        """Internally used by vectorize().
-        """
-        yedges = self.yedges[:-1] if self.sign_flipped else self.yedges[1:]
-        return np.repeat(yedges.reshape((self.y_bins(), 1)), self.x_bins(), axis=1)
-
-    def coordinates_of_center_of_bins(self):
-        """Internally used by vectorize().
-        """
-        x_centers = (self.xedges[:-1] + self.xedges[1:]) / 2
-        y_centers = (self.yedges[:-1] + self.yedges[1:]) / 2
-        xs = np.repeat(x_centers.reshape((1, self.x_bins())), self.y_bins(), axis=0)
-        ys = np.repeat(y_centers.reshape((self.y_bins(), 1)), self.x_bins(), axis=1)
-        return (xs, ys)
-
-    def centers_of_vectorize_bins(self):
-        xs, ys = self.coordinates_of_center_of_bins()
-        mask = self.vectorize_mask()
-        return np.vstack((xs[mask], ys[mask])).transpose()
-
-    def indices_for_vectorization(self):
-        """Internally used by vectorize().
-        """
-        yindices, xindices = np.indices((self.y_bins(), self.x_bins()))
-        mask = self.vectorize_mask()
-        return (yindices[mask], xindices[mask])
-
-    def value_at(self, birth, death):
-        """Internally used by vectorize().
-        """
-        y_index = np.searchsorted(self.yedges, death)
-        x_index = np.searchsorted(self.xedges, birth)
-        if not ((0 < y_index < self.yedges.size) and (0 < x_index < self.xedges.size)):
-            return None
-        return self.values[y_index-1, x_index-1]
-
-    @classmethod
-    def reconstruct_from_vector(cls, vector, histinfo):
-        """Construct a Histogram object from vector and histinfo
-
-        Args:
-        vector -- 1D ndarray of floats, this vector is translated into a histogram.
-        histinfo -- a dict that has histogram information.
-          The information is generated in vectorize_PD.save_histogram_information().
-        """
-        x_edges = np.array(histinfo["x-edges"], dtype=float)
-        y_edges = np.array(histinfo["y-edges"], dtype=float)
-        y_indices = histinfo["y-indices"]
-        x_indices = histinfo["x-indices"]
-        values = np.zeros((len(y_edges) - 1, len(x_edges) - 1), dtype=vector.dtype)
-        for (y, x, val) in zip(y_indices, x_indices, vector):
-            values[y, x] = val
-
-        return cls(values, x_edges, y_edges)
-
-    def binary_histogram_by_ranking(self, rank_range, sign=None, use_abs=True):
-        """Create a binary (bool) histogram by ranking of histogram bin values.
-
-        Args:
-        rank_range: a pair of (minimum rank, maximum rank)
-        sign: if this value is not None, only bins with positive(sign=+1) or
-              negative(sign=-1) value is selected
-        use_abs: if true, to compute a ranking, absolute value of each bin is used
-        """
-        if use_abs:
-            values = np.abs(self.values).flatten()
-        else:
-            values = self.values.flatten()
-
-        min_rank, max_rank = rank_range
-        ranks = np.argsort(values.flatten())[::-1][min_rank:max_rank - min_rank + 1]
-        mask = np.full_like(values, False, dtype=bool)
-        mask[ranks] = True
-
-        mask = mask.reshape(self.values.shape)
-        if sign is not None:
-            mask &= sign * self.values > 0
-        return BinaryHistogram(mask, self.xedges, self.yedges)
-
-
-class Ruler(object):
-    """A pair of min-max (range) and # of bins.
-    """
-    def __init__(self, minmax, bins):
-        self.minmax = minmax
-        self.bins = bins
-
-    def min(self):
-        return self.minmax[0]
-
-    def max(self):
-        return self.minmax[1]
-
-    def bin_width(self):
-        return (self.max() - self.min()) / float(self.bins)
-
-    def edges(self):
-        return np.linspace(self.minmax[0], self.minmax[1], self.bins + 1)
-
-    @staticmethod
-    def create_xy_rulers(x_range, xbins, y_range, ybins, diagram):
-        x_range = x_range or diagram.minmax_of_birthdeath_time()
-        y_range = y_range or x_range
-        ybins = ybins or xbins
-        return (Ruler(x_range, xbins), Ruler(y_range, ybins))
-
-
-class PDHistogram(Histogram):
-    """Histogram of persistence diagram
-    """
-    def __init__(self, diagram, x_ruler, y_ruler):
-        self.diagram = diagram
-        self.x_ruler = x_ruler
-        self.y_ruler = y_ruler
-        values, yedges, xedges = np.histogram2d(
-            diagram.deaths, diagram.births,
-            bins=self.yx_bins(), range=self.yx_ranges()
-        )
-        super(PDHistogram, self).__init__(values, xedges, yedges, diagram.sign_flipped)
-
-    def yx_bins(self):
-        return (self.y_ruler.bins, self.x_ruler.bins)
-
-    def yx_ranges(self):
-        return (self.y_ruler.minmax, self.x_ruler.minmax)
-
-    def apply_gaussian_filter(self, sigma):
-        """Apply a Gaussian filter to the histogram.
-
-        Note: This method works correctly only if the scale of x-axis and
-        the scale of y-axis are the same. This should be fixed in the future.
-
-        Args:
-        sigma -- standard deviation of the gaussian distribution
-        """
-        self.values = gaussian_filter(self.values, self.normalized_standard_deviation(sigma))
-
-    def normalized_standard_deviation(self, sigma):
-        """Compute the standard deviation from x-scale and sigma.
-
-        This method is internally used from apply_gaussian_filter.
-        """
-        return sigma / self.x_ruler.bin_width()
-
-    def apply_weight(self, weight_func):
-        """Apply (multiply) weights to all values in bins.
-
-        The weight is computed by weight_func.
-        Args:
-        weight_func -- an applicable object with two parameters,
-          x (birth time) and y (death time)
-        """
-        xs, ys = self.coordinates_of_center_of_bins()
-        self.values *= np.vectorize(weight_func)(xs, ys)
-
-    def multiply_histogram(self, val):
-        """Apply (multiply) a uniform weight to all values in bins.
-
-        Args:
-        val -- weight
-        """
-        self.values *= val
-
-
-class BinaryHistogram(Histogram):
-    """A PD histogram whose values are bool.
-    """
-    def filter_pairs(self, pairs):
-        return [pair for pair in pairs if self.value_at(pair.birth, pair.death)]
