@@ -14,6 +14,7 @@ class MapType(enum.Enum):
     rips = 4
     abstract = 5
 
+
 class IndexMap(object):
     """A class representing a map from index to the real birth time (called a level).
 
@@ -58,25 +59,6 @@ class IndexMap(object):
         """
         return (self.levels[pair[0]], self.levels[pair[1]])
 
-    @staticmethod
-    def load_from_json(f):
-        """Load an index map from an io object
-
-        Args:
-        f -- an io object
-        """
-        return IndexMap.load_from_dict(json.load(f))
-
-    @staticmethod
-    def load_from_jsonfile(path):
-        """Load an index map from the file whose name is path
-
-        Args:
-        path -- input file path
-        """
-        with open(path) as f:
-            return IndexMap.load_from_json(f)
-
     @staticmethod
     def load_from_dict(dict_):
         """Load an index map from the given dict object
@@ -107,7 +89,7 @@ class IndexMap(object):
             return IndexMapForAbstractFiltration(np.array(dict_["levels"]), dim,
                                                  dict_["symbols"], format_version)
 
-    FORMAT_VERSION = 3
+    FORMAT_VERSION = 4
 
 
 class IndexMapForBitmap(IndexMap):
@@ -243,9 +225,9 @@ class Simplex(object):
         self.index_map = index_map
 
     def __eq__(self, other):
-        return (isinstance(other, Simplex)
-                and self.index_map is other.index_map
-                and self.index == other.index)
+        return (isinstance(other, Simplex) and
+                self.index_map is other.index_map and
+                self.index == other.index)
 
     def __lt__(self, other):
         if not isinstance(other, Simplex):
@@ -281,7 +263,7 @@ class Simplex(object):
         if self.dim() == 0:
             return []
         s = tuple(sorted(self.index_map.simplices[self.index]))
-        return [s[:i] + s[i+1:] for i in range(len(s))]
+        return [s[:i] + s[i + 1:] for i in range(len(s))]
 
     def dim(self):
         return len(self.index_map.simplices[self.index]) - 1
