@@ -30,6 +30,8 @@ __all__ = [
     "BitmapPHTrees",  # .bitmap_phtrees
     "distance_transform",  # .distance_transform
     "BitmapOptimal1Cycle",  # .bitmap_optimal_1_cycle
+    "GraphOptimal1Cycle",  # .graph_optimal_1_cycle
+    "Optimal1Cycle",  # .optimal_1_cycle
     "example_data", "loadtxt_nd", "draw_volumes_on_2d_image",
     "draw_birthdeath_pixels_2d", "show_slice3d",
 ]
@@ -44,6 +46,7 @@ from .distance_transform import *
 from . import bitmap as bitmap
 from . import distance
 
+
 def example_data(name):
     """Returns example data.
 
