@@ -1,5 +1,6 @@
 import homcloud.pict.optimal_one_cycle as pict_opt1cyc
 import homcloud.paraview_interface as pv_interface
+import homcloud.plotly_3d as p3d
 
 
 class BitmapOptimal1Cycle(object):
@@ -99,3 +100,18 @@ class BitmapOptimal1Cycle(object):
 
     to_pvnode = to_paraview_node
     to_pvnode_for_1cycle = to_paraview_node_for_1cycles
+
+    def to_plotly3d_trace(self, color=None, name=""):
+        """
+        Constructs a plotly's trace object to visualize the optimal 1-cycle
+
+        Args:
+            color (string or None): The name of the color
+            name (string): The name of the object
+
+        Returns:
+            plotly.graph_objects.Mesh3d: Plotly's trace object
+        """
+        return p3d.Voxels(self.path(), color, name)
+
+    to_plotly3d = to_plotly3d_trace
