@@ -26,9 +26,11 @@ def argument_parser():
                         help="dimension of the input data")
     parser.add_argument("-w", "--weighted", action="store_true", default=False,
                         help="use an weighted alpha filtration")
-    parser.add_argument("--no-square", action="store_true", default=False,
+    parser.add_argument("--square", action="store_true", default=True)
+    parser.add_argument("--no-square", action="store_const", const=False,
+                        dest="square",
                         help="no squared output, if a birth radius is negative, the output is -sqrt(abs(r))")
-    parser.add_argument("--square", action="store_const", const=False, dest="no_square")
+    
     parser.add_argument("-I", "--combine-index-map", default=False, action="store_true",
                         help="Do nothing (obsoleted)")
     parser.add_argument("-D", "--convert-to-diagram", default=False, action="store_true",
@@ -103,7 +105,7 @@ def main(args=None):
     if args.partial_filtration:
         alpha_shape.become_partial_shape()
 
-    filtration = alpha_shape.create_filtration(args.no_square,
+    filtration = alpha_shape.create_filtration(args.square,
                                                load_symbols(args.vertex_symbols),
                                                args.save_boundary_map,
                                                args.save_phtrees)
