@@ -4,7 +4,9 @@ from tempfile import TemporaryDirectory
 import numpy as np
 import ripser
 
-from homcloud.pdgm_format import PDGMWriter, PDChunk, SimpleChunk, BoundaryMapChunk
+from homcloud.pdgm_format import (
+    PDGMWriter, PDChunk, SimpleChunk, BoundaryMapChunk, RipsInformationChunk, GraphWeightsChunk
+)
 from homcloud.version import __version__
 from homcloud.license import add_argument_for_license
 from homcloud.utils import load_symbols
@@ -21,8 +23,8 @@ class DistanceMatrix(object):
         self.upper_value = upper_value
         self.vertex_symbols = vertex_symbols
 
-    def build_rips_filtration(self):
-        return RipsFiltration(self.matrix, self.upper_dim, self.upper_value)
+    def build_rips_filtration(self, save_graph=False, save_cocycle=False):
+        return RipsFiltration(self.matrix.astype(np.float32), self.upper_dim, self.upper_value, save_graph, save_cocycle)
 
     def build_simplicial_filtration(self, save_boundary_map=False):
         simplices = sorted(self.all_simplices(), key=lambda x: (x[0], len(x[1])))
@@ -63,10 +65,12 @@ class DistanceMatrix(object):
 
 
 class RipsFiltration(object):
-    def __init__(self, matrix, upper_dim, upper_value):
+    def __init__(self, matrix, upper_dim, upper_value, save_graph, save_cocycle):
         self.matrix = matrix
         self.upper_dim = upper_dim
         self.upper_value = upper_value
+        self.save_graph = save_graph
+        self.save_cocycle = save_cocycle
 
     def compute_pdgm(self, f, algorithm=None, save_suppl_info=True):
         algorithm = algorithm or self.favorite_algorithm()
@@ -100,16 +104,45 @@ class RipsFiltration(object):
         check_upper_value_should_be_infinity()
         diagrams = ripser.ripser(
             self.matrix, self.upper_dim, self.upper_value, 2,  # Z/2Z
-            True  # The first argument is a distane matrix, not a pointcloud
-        )["dgms"]
+            True,  # The first argument is a distane matrix, not a pointcloud
+            do_cocycles=self.save_cocycle
+        )
 
         writer = PDGMWriter(f, "rips", self.upper_dim + 1)
 
-        for d, pairs in enumerate(diagrams):
+        for d, pairs in enumerate(diagrams["dgms"]):
             writer.append_chunk(build_pdchunk(d, pairs))
 
+        self.append_rips_information_chunk(writer)
+        self.append_graph_weights_chunk(writer)
+
+        if self.save_cocycle:
+            self.append_cocycle_chunk(writer,
+                                      diagrams["dgms"],
+                                      diagrams["cocycles"])
+
         writer.write()
 
+    def append_graph_weights_chunk(self, writer):
+        if not self.save_graph:
+            return
+        writer.append_chunk(GraphWeightsChunk(self.matrix))
+
+    def append_rips_information_chunk(self, writer):
+        writer.append_chunk(RipsInformationChunk(self.num_vertices))
+
+    def append_cocycle_chunk(self, writer, dgms, cocycles):
+        for d in range(len(dgms)):
+            writer.append_simple_chunk("cocycles", [
+                cocycle.tolist() for (cocycle, pair)
+                in zip(cocycles[d], dgms[d])
+                if pair[1] != np.inf
+            ], d)
+
+    @property
+    def num_vertices(self):
+        return self.matrix.shape[0]
+
     def compute_pdgm_by_dipha(self, f):
         pd_chunks = [PDChunk("pd", d, [], [], []) for d in range(self.upper_dim + 1)]
 
@@ -127,17 +160,19 @@ class RipsFiltration(object):
 
         writer = PDGMWriter(f, "rips", self.upper_dim + 1)
         writer.extend_chunks(pd_chunks)
+        self.append_rips_information_chunk(writer)
+        self.append_graph_weights_chunk(writer)
         writer.write()
 
     def write_dipha_complex(self, output):
-        num_points = self.matrix.shape[0]
+        num_vertices = self.matrix.shape[0]
 
         def write_dipha_header():
-            output.write(struct.pack("qqq", 8067171840, 7, num_points))
+            output.write(struct.pack("qqq", 8067171840, 7, num_vertices))
 
         def write_matrix():
-            for k in range(num_points):
-                for l in range(num_points):
+            for k in range(num_vertices):
+                for l in range(num_vertices):
                     output.write(struct.pack("d", self.matrix[k, l] * 2))
 
         write_dipha_header()
@@ -150,6 +185,7 @@ class RipsFiltration(object):
 class SimplicialFiltration(object):
     def __init__(self, dmatrix, index_to_level, index_to_simplex,
                  simplex_to_index, vertex_symbols, save_boundary_map=False):
+
         self.distance_matrix = dmatrix
         self.index_to_level = index_to_level
         self.index_to_simplex = index_to_simplex
