@@ -15,6 +15,7 @@ import homcloud.pict.pict3d_vtk as pict3d_vtk
 from homcloud.version import __version__
 
 import matplotlib
+
 matplotlib.rcParams['backend'] = "Agg"
 
 import matplotlib.pyplot as plt  # NOQA
@@ -30,11 +31,14 @@ def self_check(args):
     self_check_binary_2D_bitmap()
     self_check_binary_periodic_2D_bitmap()
     self_check_rips()
-    self_check_rips_maxvalue()
+    if "--no-dipha" not in args:
+        self_check_rips_maxvalue()
     self_check_plot_pd()
     self_check_optimal_volume()
     if "--no-paraview" not in args:
         self_check_paraview()
+    if "--plotly" in args:
+        self_check_plotly()
 
 
 def print_version():
@@ -59,8 +63,8 @@ def self_check_alpha_shape_3_with_weight():
 
 def self_check_alpha_shape_3_periodic_with_weight():
     print('Periodic Alpha Shape 3 with weights ... ', end='')
-    pc = np.random.uniform(0, 1, (100, 3))
-    weight = np.random.uniform(0.001, 0.05, 100)
+    pc = np.random.uniform(0, 1, (200, 3))
+    weight = np.random.uniform(0.001, 0.05, 200)
     hc.PDList.from_alpha_filtration(
         pc, weight=weight, periodicity=[(0, 1), (0, 1), (0, 1)]
     )
@@ -140,7 +144,7 @@ def self_check_optimal_volume():
 
 def self_check_paraview():
     print('Paraview path: {0} ({1})'.format(utils.paraview_programname(),
-                                           shutil.which(utils.paraview_programname())))
+                                            shutil.which(utils.paraview_programname())))
     print('Paraview fake invoke ... ', end='')
     with unittest.mock.patch('subprocess.check_call'):
         with unittest.mock.patch('subprocess.Popen'):
@@ -168,5 +172,32 @@ def self_check_paraview():
     print('ok')
 
 
+def self_check_plotly():
+    import plotly.graph_objects as go
+    import homcloud.plotly_3d as p3d
+    print("Plotly 3D drawing (3D objects are displayed in your browser, check it and close the tab) ...",
+          end="")
+    vertices = [
+        [0, 0, 0], [1, 0, 2], [-0.7, 1.4, 1.9], [-0.7, -1.4, 2.1]
+    ]
+    tetrahedron = [[vertices[0], vertices[1], vertices[2]],
+                   [vertices[0], vertices[1], vertices[3]],
+                   [vertices[0], vertices[2], vertices[3]],
+                   [vertices[1], vertices[2], vertices[3]]]
+    loop = [[np.cos(2 * np.pi / 6 * n), np.sin(2 * np.pi / 6 * n), 0] for n in range(6)]
+    bitmap3d = np.array([[[0, 0, 0], [0, 1, 0], [0, 1, 0]],
+                         [[1, 1, 1], [1, 1, 1], [1, 1, 0]]])
+    fig = go.Figure(data=[
+        p3d.Simplices(tetrahedron, color="black", width=2, name="tetrahedron"),
+        p3d.SimplicesMesh(tetrahedron, color="blue"),
+        p3d.Loop(loop, color="green", width=4, name="loop"),
+        p3d.Bitmap3d(bitmap3d, color="green"),
+    ], layout=dict(scene=p3d.SimpleScene()))
+    fig.update_traces(opacity=0.5, selector=dict(type="mesh3d"))
+    fig.show(renderer="browser")
+
+    print(" ok")
+
+
 if __name__ == '__main__':
     self_check(sys.argv[1:])
