@@ -126,6 +126,7 @@ class ParaViewDrawer(object):
         self.output_points(f)
         self.output_vertices(f)
         self.output_lines(f)
+        self.output_polygons(f)
 
     def output_vertices(self, f):
         if not self.vertices:
@@ -137,6 +138,12 @@ class ParaViewDrawer(object):
             f.write("1 {}\n".format(v))
         f.write("\n")
 
+    def output_lines(self, f):
+        pass
+
+    def output_polygons(self, f):
+        pass
+
     def output_line_colors(self, f):
         f.write("CELL_DATA {}\n".format(len(self.colors)))
         f.write("SCALARS colors float 1\n")
@@ -288,10 +295,6 @@ class ParaViewSparseCubeDrawer(ParaViewDrawer):
                     self.draw_line(indices[tuple(dv)], indices[tuple(dv + dl)], color,
                                    **threshold_values)
 
-    def draw_cell(self, cell, cube_geom_resolver, color, **threshold_values):
-        coord, non_deg = cube_geom_resolver.resolve_cell(cell)
-        self.draw_cube(coord, non_deg, color, **threshold_values)
-
     def prepare_points(self, coord, non_deg):
         dvs = ParaViewCubeDrawer.dvs(non_deg)
         indices = np.zeros([2] * self.ndim, dtype=int)
@@ -315,6 +318,7 @@ class ParaViewSparseCubeDrawer(ParaViewDrawer):
             f.write("2 {} {}\n".format(line[0], line[1]))
         f.write("\n")
 
+
 class ParaViewSparseBitmapDrawer(ParaViewDrawer):
     def __init__(self, n_colors, column_names):
         super().__init__(n_colors, column_names)
@@ -418,6 +422,27 @@ class ParaViewPolyLineDrawer(ParaViewDrawer):
         return len(self.points)
 
 
+class ParaView3DSurfaceDrawer(ParaViewDrawer):
+    def __init__(self, n_colors, surface, column_names):
+        super().__init__(n_colors, column_names)
+        self.num_triagnles = len(surface)
+        self.surface = surface
+        for _ in range(self.num_triagnles):
+            self.append_attributes(0, {})
+
+    def output_points(self, f):
+        points = np.array(self.surface).reshape(3 * self.num_triagnles, 3)
+        f.write("POINTS {} double\n".format(self.num_triagnles * 3))
+        for point in points:
+            f.write("{} {} {}\n".format(point[0], point[1], point[2]))
+        f.write("\n")
+
+    def output_polygons(self, f):
+        f.write("POLYGONS {} {}\n".format(self.num_triagnles, self.num_triagnles * 4))
+        for k in range(self.num_triagnles):
+            f.write("3 {} {} {}\n".format(3 * k, 3 * k + 1, 3 * k + 2))
+
+
 def boundary_of_connected_simplices(simplices):
     def boundaries(key):
         if len(key) == 1:
